@@ -4,6 +4,11 @@ use strict;
 use warnings;
 no warnings 'uninitialized';
 
+use lib '/home/wolf/proj/Yote/YoteBase/lib';
+use lib '/home/wolf/proj/Yote/ServerYote/lib';
+use lib '/home/wolf/proj/Yote/LockServer/lib';
+use lib '/home/wolf/proj/Yote/CCCC/lib';
+
 use Yote;
 use Yote::Server;
 
