@@ -41,26 +41,29 @@ yote.init = function( args ) {
 
     var yoteServerURL = args.yoteServerURL || '';
 
-    var token, root;
+    var token, root, app, appname, acct;
 
-    if( typeof sessionStorage !== 'undefined' ) {
-        token = sessionStorage.getItem( 'token' );
-    }
+
+    token = localStorage.getItem('token');
     
     // cache storing objects and their meta-data
     var class2meths = {};
-    var id2obj = {};
 
-    // returns an object, either the cache or server
-    yote.fetch = function( id ) {
-        var r = id2obj[ id ];
-        if( typeof r === 'undefined' ) {
-            console.warn( "warning : fetching asynchronously" );
-            r = root.fetch( id );
-        }
-        return r;
+    var id2obj = {};
+    
+
+    var _register = function( id, obj ) {
+        id2obj[ id ] = obj;
+        console.log( [ "REGGY " + id, obj._data ] );
+        localStorage.setItem( id, JSON.stringify( {
+            id    : obj.id,
+            _cls  : obj._cls,
+            _data : obj._data,
+            _meths : obj._meths
+        } ) );
     }
 
+
     // creates a proxy method that contacts the server and
     // returns data
     function makeMethod( mName ) {
@@ -73,7 +76,7 @@ yote.init = function( args ) {
                 failhandler = handler;
                 handler = data;
                 data = [];
-            } else if( typeof data !== 'object' && typeof data !== 'undefined' ) {
+            } else if( ! Array.isArray( data ) ) {
                 // TODO - maybe detect if there are non-functions in the arguments list? gather into array all non-function args
                 // like so
                 data = [data];
@@ -83,45 +86,49 @@ yote.init = function( args ) {
         };
     };
 
-
-    // method for translating and storing the objects
-    function makeObj( datastructure ) {
-        /* method that returns the value of the given field on the yote obj */
-        var obj = id2obj[ datastructure.id ];
-        var isUpdate = typeof obj === 'object';
-        // TODO : maybe include what was updated and pass that to the action listener?
-        if( ! isUpdate ) {
-            obj = {
-                id        : datastructure.id,
-                listeners : []
-            };
-            id2obj[ datastructure.id ] = obj;
-        }
-        obj._cls  = datastructure.cls;
-        obj._data = datastructure.data;
-
-        // takes a function that takes this object as a
-        // parameter
-        obj.addUpdateListener = function( listener ) {
-            obj.listeners.push( listener );
-            return this;
-        }
-        obj.removeUpdateListeners = function() {
-            obj.listeners = [];
-            return this;
-        }
-        obj.get = function( key ) {
-            var val = this._data[key];
-            if( typeof val === 'undefined' || val === null ) {
-                return undefined;
+    var makeObjSkell = function( cls ) {
+        var obj = {
+            _cls  : cls,
+            _data : {},
+            listeners : {},
+            
+            // takes a function that takes this object as a
+            // parameter
+            addUpdateListener : function( listener, key ) {
+                key = key || '_';
+                if( key === '_' ) {
+                    var defs = this.listeners[ key ];
+                    if( ! defs ) {
+                        defs = [];
+                        this.listeners[ key ] = defs;
+                    }
+                    defs.push( listener );
+                } else {
+                    this.listeners[ key ] = listener;
+                }
+                return this;
+            },
+            removeUpdateListeners : function( key ) {
+                if( key ) {
+                    delete this.listeners[ key ];
+                } else {
+                    this.listeners = {};
+                }
+                return this;
+            },
+            get : function( key ) {
+                var val = this._data[key];
+                if( typeof val === 'undefined' || val === null ) {
+                    return undefined;
+                }
+                if( typeof val === 'string' && val.startsWith( 'v' ) ) {
+                    return val.substring( 1 );
+                } 
+                return yote.fetch( val );
             }
-            if( typeof val === 'string' && val.startsWith( 'v' ) ) {
-                return val.substring( 1 );
-            } 
-            return yote.fetch( val );
         };
 
-        if( datastructure.cls === 'ARRAY' ) {
+        if( cls === 'ARRAY' ) {
             obj.toArray = function() {
                 var a = [];
                 for( var k in obj._data ) {
@@ -138,21 +145,63 @@ yote.init = function( args ) {
                 return Object.keys( obj._data ).length;
             };
         }
-        
-        var mnames = class2meths[ datastructure.cls ] || [];
+        var mnames = class2meths[ cls ] || [];
+        obj._meths = mnames;
         mnames.forEach( function( mname ) {
             obj[ mname ] = makeMethod( mname );
         } );
+        return obj;
+    }; //makeObjSkell
+    
+    // returns an object, either the cache or server
+    var _fetch = function( id ) {
+        if( id2obj[id] ) {
+            return id2obj[id];
+        }
+        var objdata = JSON.parse( localStorage.getItem( id ) );
+        if( ! objdata ) {
+            return null;
+        }
+        var obj = makeObjSkell( objdata._cls );
+        obj._data = objdata._data;
+        obj.id = id;
+        id2obj[id] = obj;
+        
+        return obj;
+    }; //_fetch
 
+    yote.fetch = _fetch;
+    
+    // method for translating and storing the objects
+    // NOTE : returns a function used for updating
+    function makeObj( datastructure ) {
+        /* method that returns the value of the given field on the yote obj */
+        var obj = _fetch( datastructure.id );
+        var isUpdate = obj !== null && typeof obj === 'object';
+        // TODO : maybe include what was updated and pass that to the action listener?
+        if( ! isUpdate ) {
+            obj = makeObjSkell( datastructure.cls );
+            obj.id = datastructure.id;
+            obj._data = datastructure.data;
+        }
+        console.log( [ "MAKE OBJ " + datastructure.id, obj, obj._data ] );
+        obj._data = datastructure.data;
+        _register( datastructure.id, obj );
+        
         // fire off an event for any update listeners
         return function() {
             if( isUpdate ) {
-                for( var i in obj.listeners ) {
-                    obj.listeners[i]( obj );
+                for( var key in obj.listeners ) {
+                    if( key === '_' ) {
+                        obj.listeners[key].forEach( function( l ) {
+                            l( obj );
+                        } );
+                    } else {
+                        obj.listeners[key]( obj );
+                    }
                 };
             }
         }
-
     } //makeObj
     
     function processReturn( returnData ) {
@@ -177,11 +226,16 @@ yote.init = function( args ) {
         }
     } //processReturn
 
-    function processRaw(rawResponse) {
-        if( ! rawResponse ) {
+    function processRaw(rawResponse,succHandle,failHandle) {
+        var res = JSON.parse( rawResponse );
+
+        // check for errors
+        if( res.err ) {
+            if( failHandle ) {
+                failHandle( res.err );
+            }
             return;
         }
-        var res = JSON.parse( rawResponse );
         
         // ** 3 parts : methods, updates and result
         
@@ -194,24 +248,25 @@ yote.init = function( args ) {
         
         // updates
         if( res.updates ) {
-            var makeFuns = [];
+            var makeOrUpdateFuns = [];
             res.updates.forEach( function( upd ) {
                 if( typeof upd !== 'object' || ! upd.id ) {
                     console.warn( "Update error, was expecting object, not : '" + upd + "'" );
                 } else {
                     // good place for an update listener
-                    makeFuns.push( makeObj( upd ) );
+                    makeOrUpdateFuns.push( makeObj( upd ) );
                 }
             } ); //updates section
-            makeFuns.map( function( fun ) {
+            makeOrUpdateFuns.map( function( fun ) {
                 fun();
             } );
         }
         
         // results
-        if( res.result ) {
+        if( res.result && succHandle ) {
             var resses = processReturn( res.result );
-            return resses.length > 1 ? resses : resses[0];
+            var finalRes = resses.length > 1 ? resses : resses[0];
+            succHandle( finalRes );
         }
     }; //processRaw
 
@@ -220,11 +275,8 @@ yote.init = function( args ) {
     function reqListener( succHandl, failHandl ) { 
         return function() {
             console.log( "GOT FROM SERVER : " + this.response );
-            if( this.response ) {
-                var returnVal = processRaw( this.response );
-                if( succHandl ) {
-                    succHandl( returnVal );
-                }
+            if( typeof this === 'object' ) {
+                processRaw( this.response, succHandl, failHandl );
             } else if( failHandl ) {
                 failHandl( 'failed' );
             }
@@ -235,14 +287,14 @@ yote.init = function( args ) {
         if( typeof obj !== 'object' ) {
             return typeof obj === 'undefined' || obj === null ? undefined : 'v' + obj;
         }
-        if( id2obj[ obj.id ] === obj ) {
+        if( _fetch( obj.id ) === obj ) {
             return obj.id;
         }
 
         for( var idx in obj ) {
             var v = obj[idx];
             if( typeof v === 'object' ) {
-                if( id2obj[ v.id ] === v ) {
+                if( _fetch( v.id ) === v ) {
                     v = v.id;
                 } else {
                     v = readyObjForContact( v );
@@ -255,29 +307,7 @@ yote.init = function( args ) {
         return obj;
     }
 
-    function contact_worker(id,action,data,handl,errhandl) {
-        worker = new SharedWorker( "/__/js/yote.worker.js" );
-        worker.port.start();
-
-        var readiedData = typeof data === 'undefined' || data === null ? undefined : readyObjForContact( data );
-
-        // for a single parameter, wrap into a parameter list
-        var sendData = JSON.stringify(  readiedData );
-        worker.port.onmessage = function(e) {
-            if( e.data ) {
-                var retVal = processRaw( e.data );
-                if( handl ) {
-                    handl( e.data );
-                }
-            } else if( errhandl ) {
-                errhandl( 'failed' );
-            }
-        }
-
-        worker.port.postMessage( [ id, action, readiedData ] );
-    }
-
-    function contact_server(id,action,data,handl,errhandl) { 
+    function contact(id,action,data,handl,errhandl) { 
         var oReq = new XMLHttpRequest();
         oReq.addEventListener("loadend", reqListener( handl, errhandl ) );
         oReq.addEventListener("error", function(e) { alert('error : ' + e) } );
@@ -307,19 +337,8 @@ yote.init = function( args ) {
         oReq.setRequestHeader("Content-type", "application/x-www-form-urlencoded" );
         oReq.send( sendData );
 
-    }; // contact_server
-
-    var contact, worker, init_root_args = [];
-    if( args.contact == 'worker'  ) {
-        contact = contact_worker;
-        if( args.appFile ) {
-            init_root_args.push( args.appFile );
-        }
-    } else {
-        contact = contact_server;
-    }
-    
-    // translates text to objects
+    }; // contact
+   // translates text to objects
     function xform_in( item ) {
         if( typeof item === 'object' ) {
             if( item === null ) {
@@ -339,7 +358,7 @@ yote.init = function( args ) {
             if( typeof item === 'string' && item.startsWith('v') ) {
                 return item.substring( 1 );
             } else {
-                return id2obj[ item ];
+                return _fetch( item );
             }
         }
     }
@@ -351,7 +370,7 @@ yote.init = function( args ) {
             if( Array.isArray( res ) ) {
                 return res.map( function( x ) { return xform_out( x ) } );
             }
-            var obj = id2obj[ res.id ];
+            var obj = _fetch( res.id );
             if( obj ) { return res.id }
             var ret = {};
             for( var key in res ) {
@@ -364,25 +383,25 @@ yote.init = function( args ) {
         return 'v' + res;
     }//xform_out
 
-    var appname = args.appName;
+    appname = args.appName;
     var handler = args.handler;
     var errhandler = args.errHandler;
 
     if( ! handler ) {
         console.warn( "Warning : yote.init called without handler" );
     }
-    contact( '_', 'init_root', init_root_args, function(res) {
+    contact( '_', 'init_root', [], function(res) {
         root  = res[0];
         token = res[1];
-        if( typeof sessionStorage !== 'undefined' ) {
-            sessionStorage.setItem( 'token', token );
-        }
+        localStorage.setItem( 'token', token );
+
         if( handler ) {
             if( appname ) {
                 root.fetch_app( [appname], function( result ) {
                     if( Array.isArray( result ) ) {
-                        var app = result[0];
-                        var acct = result[1];
+                        console.log( [ result,"RESSY" ] );
+                        app = result[0];
+                        acct = result[1];
                     } else {
                         app = result;
                     }
@@ -394,6 +413,42 @@ yote.init = function( args ) {
         }
     }, errhandler );
 
+    yote.logout = function( handler ) {
+        if( app ) {
+            app.logout(function() {
+                localStorage.clear();
+
+                acct = undefined;
+                token = undefined;
+                app = undefined;
+
+                contact( '_', 'init_root', [], function(res) {
+                    root  = res[0];
+                    token = res[1];
+                    localStorage.setItem( 'token', token );
+
+                    if( appname ) {
+                        root.fetch_app( [appname], function( result ) {
+                            if( Array.isArray( result ) ) {
+                                app = result[0];
+                            } else {
+                                app = result;                                
+                            }
+                            if( handler ) {
+                                handler( root, app );
+                            }
+                        } );
+                    } else if( handler ) {
+                        handler( root );
+                    }
+                }, errhandler );
+
+            }, function(err) { localStorage.clear(); } );
+        } else {
+            localStorage.clear();
+        }
+    } //yote.logout
+    
 }; //yote.init
 
 yote.sameyoteobjects = function( a, b ) {
